<?php

namespace Drupal\commerce_aqayepardakht\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

class PaymentOffsiteForm extends BasePaymentOffsiteForm
{

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state)
  {
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    $order = $payment->getOrder();
    $order_id = $order->id();
	$store = $order->getStore()->label();
    $payer_email = $order->getEmail();
	
    $redirect = Url::fromUri('base:/checkout/' . $order_id . '/payment/return/', ['absolute' => TRUE])
      ->toString();
    $amount = (int)$payment->getAmount()->getNumber();
    if ($payment->getAmount()->getCurrencyCode() !== 'IRR') {
      // Converts Iranian Toman to Rials,
      // because aqayepardakht payment service accepts IRR
      $amount = $amount * 10;
    }
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $gateway_configuration = $payment_gateway_plugin->getConfiguration();
    $api_key = $gateway_configuration['api_key'];

    if ($gateway_configuration["mode"] == "test") {
      $api_key = 'aqayepardakht';
    }

    try {

      $params = [
        "pin" => $api_key,
        "amount" => $amount,
        "callback" => $redirect,
        "invoice_id" => $order_id,
		"description" => "پرداخت در فروشگاه : " . $store . " | ایمیل خریدار : " . $payer_email
      ];

      $params = json_encode( $params );
      $ch = curl_init( 'https://panel.aqayepardakht.ir/api/create' );
      curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
      curl_setopt( $ch, CURLINFO_HEADER_OUT, true );
      curl_setopt( $ch, CURLOPT_POST, true );
      curl_setopt( $ch, CURLOPT_POSTFIELDS, $params );

      curl_setopt( $ch, CURLOPT_HTTPHEADER, array(
        'Content-Type: application/json',
        'Content-Length: ' . strlen( $params ) ) );
      $result = curl_exec( $ch );
      curl_close( $ch );

      if ($result && !is_numeric($result)) {
      header('Location: https://panel.aqayepardakht.ir/startpay/' . $result);

      } else {
        drupal_set_message('Error: ' . implode("<br>",$result), 'error');
      }

    } catch (\Exception $e) {
      drupal_set_message('EError: ' . $e->getMessage(), 'error');
    }

    $checkout_review_page = Url::fromUri('base:/checkout/' . $order_id . '/review', ['absolute' => TRUE])
      ->toString();
    return $this->buildRedirectForm($form, $form_state, $checkout_review_page, [], NULL);

  }
}
